DEFAULT_LANGUAGE = "ja"

DEFAULT_OUT_AUDIO_DURATION = 60.0 * 1  # seconds

DEFAULT_COMP_AUDIO_DURATION = 60.0 * 20  # seconds
DEFAULT_SR = 48000  # Hz

DEFAULT_CHUNK_DURATION = 0.5  # seconds
DEFAULT_BUFFER_DURATION = 0.25  # seconds

DEFAULT_TARGET_DB = -30.0  # dBFS
DEFAULT_THRESHOLD_DB = -70.0  # dBFS

APP_NAME = "waddle"
APP_AUTHOR = "emptymap"
